//
// Copyright (c) 2012 Secret Geometry, Inc.. All rights reserved.
//

#ifndef CELLGRID_HPP
#define CELLGRID_HPP

#include "Cell.hpp"
#include "Types.hpp"
#include "Utilities.hpp"

namespace term {

// 
// CellGrid
// 

class CellGrid
{
  public:
	
    // 
    CellGrid(int width, int height);
    
    // 
    int width() const;
	int height() const;
    
    // 
    CellCursor begin() const;
	
    // 
    CellCursor end() const;
    
    //
    int resize(int newWidth, int newHeight);
    
    //
    void addRows(int n);
    
    // 
    int dataBetweenCursors(CellCursor from, CellCursor to, Cell* buffers[], int lengths[]);
    
    // 
    int dataBetweenRows(int fromRow, int toRow, Cell* buffers[], int lengths[]);
    
    //
    const Cell read(CellCursor c);
    
    //
    void write(CellCursor c, const Cell& value);
    
    //
    void clear();
    
    //
    void fill(CellCursor cursor, CellCursor endCursor, const Cell& value = Cell());
    
    //
    void move(CellCursor from, CellCursor fromEnd, CellCursor to, const Cell& clearValue = Cell());
    
    //
    CellCursor shiftCursor(CellCursor c, int distance);
    
    //
    int distanceBetweenCursors(CellCursor c1, CellCursor c2);
    
  private:
    
    typedef std::vector<Cell>::iterator Iterator;
    
    Iterator iteratorFromCursor(CellCursor c);
    
    bool viewRowRangeIsValid(int beginRow, int endRow);
    
    void safeCopy(Iterator source, Iterator sourceEnd, Iterator target);
    
    void safeCopyBackward(Iterator source, Iterator sourceEnd, Iterator targetEnd);
    
    void safeFill(Iterator begin, Iterator end, const Cell& value = Cell());
    
    int viewWidth;
    int viewHeight;
    std::vector<Cell> buffer;
};

} // namespace term

#endif // CELLGRID_HPP
