//
// Copyright (c) 2012 Secret Geometry, Inc.. All rights reserved.
//

#ifndef CELLGRIDBUFFER_HPP
#define CELLGRIDBUFFER_HPP

#include "Cell.hpp"
#include "Types.hpp"
#include "Utilities.hpp"

namespace term {

// 
// CellGridBuffer
// 

class CellGridBuffer
{
  public:
	
    // 
    CellGridBuffer(int width, int height, int maxRows);
    
    // 
    int width() const;
	int height() const;
    
    // 
    int activeRows() const;
    
    // 
    int currentRows();
    
    // 
    int passedRows();
    
    // 
    int seekRowOffset() const;
    
    // 
    CellCursor currentBegin() const;
	
    // 
    CellCursor end() const;
    
    //
    const Cell read(CellCursor c);
    
    // 
    int dataBetweenRows(int fromRow, int toRow, Cell* buffers[], int lengths[]);
    
    //
    int resizeView(int newWidth, int newHeight, int keepRow);
    
    //
    void resizeRowCapacity(int newCapacity);
    
    //
    void addRows(int n);
    
    //
    void seekRows(int n);
    
    //
    void clearCurrent();
    
    //
    void clearToLine(int line);
    
    //
    void write(int x, int y, const Cell& value);
    
    //
    void write(CellCursor c, const Cell& value);
    
    //
    void fill(CellCursor cursor, CellCursor endCursor, const Cell& value = Cell());
    
    //
    void move(CellCursor from, CellCursor fromEnd, CellCursor to, const Cell& clearValue = Cell());
    
    //
    void sanityCheck();
    
    //
    CellCursor shiftCursor(CellCursor c, int distance);
    
    //
    int distanceBetweenCursors(CellCursor c1, CellCursor c2);
    
  private:
    
    typedef std::vector<Cell>::iterator Iterator;
    
    Iterator iteratorFromCursor(CellCursor c);
    
    int dataRowForViewRow(int viewRow);
    
    bool viewRowRangeIsValid(int beginRow, int endRow);
    
    bool dataIsSplitBetweenRows(int fromRow, int toRow);
    
    void safeCopy(Iterator source, Iterator sourceEnd, Iterator target);
    
    void safeCopyBackward(Iterator source, Iterator sourceEnd, Iterator targetEnd);
    
    void safeFill(Iterator begin, Iterator end, const Cell& value = Cell());
    
    int viewWidth;
    int viewHeight;
    int rowCapacity;
    int dataBeginRow;
    int dataEndRow;
    int streamOffset;
    std::vector<Cell> buffer;
};

} // namespace term

#endif // CELLGRIDBUFFER_HPP
