//
// Copyright (c) 2012 Secret Geometry, Inc.. All rights reserved.
//

#ifndef CELL_HPP
#define CELL_HPP

#include "Types.hpp"
#include "Unicode.hpp"

namespace term {
    
// 
// Cell
//
// A Cell packs a char, a style, and two colors into four unsigned bytes. The first byte contains the char, 
// and the next two bytes hold a foreground and background color index.  Foreground indices can be in the range [0, 16),
// while background indices are limited to [0, 8).  The top eight foreground indices are extra bright versions of the 
// lower eight and are intended for bold text. Reverse style simply swaps the (non-bold) foreground and background 
// colors.
// 

class Cell
{
  public:
    
    Cell(uint32_t c = '\0');
    
    uint32_t getCharacter() const;
    void setCharacter(uint32_t c);
    
    UnicodeRangeEnum getUnicodeRange() const;
    uint8_t getCharacterOffset() const;
    
    uint8_t getFgColor() const;
    void setFgColor(uint8_t index);
    
    uint8_t getBgColor() const;
    void setBgColor(uint8_t index);
    
    void bold();
    void unbold();
    
    void reverse();
    void noReverse();
    
    void underscore();
    void noUnderscore();
    
    void blink();
    void noBlink();
    
    void dim();
    bool isDim() const;
    
    void resetStyle();
    
  private:
	
	uint8_t values[4];
};

} // namespace term

#endif // CELL_HPP
