//
// Copyright (c) 2012 Secret Geometry, Inc.. All rights reserved.
//

#ifndef PARSER_HPP
#define PARSER_HPP

#include "Dispatcher.hpp"
#include "Utilities.hpp"

namespace term {

// 
// Parser
// 

class Parser
{
  public:
    
    Parser(Dispatcher& d);
    
    void parse(const uint8_t* str, int count);
    void parse(uint8_t c);
    
  private:
	
    void parseTextChar(uint8_t c);
    void parseUnicodeTextChar(uint8_t c);    
	void parseEscapeSequenceChar(uint8_t c);    
	void parseCharacterSetSequenceChar(uint8_t c);    
    void parseLineAttributeSequenceChar(uint8_t c);
    void parseControlSequenceChar(uint8_t c);
    void parseOperatingSystemCommandChar(uint8_t c);
    
    void printChar(uint8_t c);
	
	Dispatcher& dispatcher;
    EscapeArgs escapeArgs;
    ControlArgs controlArgs;
    UInt8Vector osCommand;
    uint8_t characterSetType;
    uint32_t codePoint;
    int remainingBytesInChar;
    bool invalidSequence;
    bool hasQuestionMark;
    bool wrapOnControlChar;
    
    enum
    {
        TEXT,
        UNICODE_TEXT,
        ESCAPE_SEQUENCE,
        CHARACTER_SET_SEQUENCE,
        LINE_ATTRIBUTE_SEQUENCE,
        CONTROL_SEQUENCE,
        OPERATING_SYSTEM_COMMAND
    }
    mode;
};

} // namespace term

#endif // PARSER_HPP
