//
// Copyright 2012 Secret Geometry, Inc.  All rights reserved.
//

#ifndef TERMINAL_HPP
#define TERMINAL_HPP

#include "Config.hpp"
#include "Types.hpp"
#include "Context.hpp"
#include "Dispatcher.hpp"
#include "Parser.hpp"
#include "Pty.hpp"

#ifdef USE_SSH
    #include "Ssh.hpp"
#endif

namespace term {

//
// Terminal
//
// A Terminal acts as a facade for a number of private components and handles the data flow between them.
//

class Terminal
{
  public:
    
    // 
    Terminal(int width,
             int height,
             int scrollbackHeight,
             BeepCallback beepFunc,
             ResizeCallback resizeFunc,
             const char* workingDir,
             const char* language,
             const char* termIdentifier,
             const char* cathodeIdentifier);
    
    // 
    ~Terminal();
    
    //
    Context& getContext();
	
    //
    void update(int numBytes);
    
    //
	void clearBuffer();
    
    //
    void write(uint8_t c);
    void write(const char* s);
    void write(const uint8_t* s, size_t n);
    
    // 
    void upArrowKey();
    void downArrowKey();
    void rightArrowKey();
    void leftArrowKey();
	void forwardDeleteKey();
    void pageUpKey();
    void pageDownKey();
    void functionKey(int index);
    
    void moveWordLeft();
    void moveWordRight();
    
    //
    void resize(int cols, int rows);
    
    // 
    int columnCount();
    void setColumnCount(int cols);
    
    // 
    int rowCount();
    void setRowCount(int rows);
    
    //
    int cursorColumn();
    int cursorRow();
    
    // 
	bool dirty();
	void setDirty(bool b);
    
    // 
    const char* activeProcessName();
    
    // 
    int processCount() const;
    
    // 
    bool isRunning();
	
  private:
	
	Context context;
    Dispatcher dispatcher;
    Parser parser;
    uint8_t* buffer;
    bool edited;
    
    #ifdef USE_SSH
        Ssh pty;
    #else
        Pty pty;
    #endif
};

} // namespace term

#endif // TERMINAL_HPP
