//
//  Types.hpp
//  TermKit
//
//  Created by Colin Caufield on 2012-06-25.
//  Copyright (c) 2012 Secret Geometry. All rights reserved.
//

#ifndef TERMKIT_TYPES_HPP
#define TERMKIT_TYPES_HPP

#include <stdint.h>
#include <vector>
#include <set>

namespace term {


class Cell;
class CellGrid;
class CellGridBuffer;
class Context;
class Dispatcher;
class Parser;
class Pty;
class Ssh;


typedef std::set<int> IntSet;
typedef std::set<int>::iterator IntSetItr;
typedef std::vector<uint8_t> UInt8Vector;
typedef UInt8Vector EscapeArgs;

typedef void (*BeepCallback)();
typedef void (*ResizeCallback)(Context*, int, int);


enum EraseInLineMode
{
    CURSOR_TO_FAR_RIGHT,
    FAR_LEFT_TO_CURSOR,
    ENTIRE_LINE
};

enum EraseInDisplayMode
{
    CURSOR_TO_END,
    BEGINNING_TO_CURSOR,
    ENTIRE_GRID
};

enum Mode
{
    COLUMN = 3,
    INSERT = 4,
    ORIGIN = 6,
    AUTO_WRAP = 7,
    LINE_FEED = 20,
    AUTO_REVERSE_WRAP = 45,
    ALT_SCREEN_BUFFER = 47,
    ALT_SCREEN_BUFFER_AND_CLEAR = 1049,
};

enum Request
{
    NO_REQUEST,
    DEVICE_ATTRIBUTES_REQUEST,
    CURSOR_POSITION_REQUEST
};


class ControlArgs : public std::vector<int>
{
  public:
    
    // Return 0 for out-of-range indices.
    int operator[](unsigned int n) const {return (n < size()) ? at(n) : 0;}
};

struct CellCursor
{
    int x, y;
    
    CellCursor() : x(0), y(0) {}
    
    CellCursor(int i, int j) : x(i), y(j) {}
    
    bool operator==(const CellCursor& other) const
    {
        return x == other.x && y == other.y;
    }
};
    
} // namespace term

#endif
