//
//  Unicode.h
//  Cathode
//
//  Created by Colin Caufield on 2012-07-17.
//

#ifndef UNICODE_HPP
#define UNICODE_HPP

#include <stdint.h>

#define UNICODE_RANGE_LENGTH 0x100

enum UnicodeRangeEnum
{
    // U+0000...U+0100 - Basic Latin, Latin-1 Supplement.
    UnicodeRange_Latin = 0,
    
    // U+0100...U+0200 - Latin Extended-A, Latin Extended-B...
    UnicodeRange_LatinExt1 = 1,
    
    // U+0200...U+0300 - ...Latin Extended-B, IPA Extensions, Spacing Modifier Letters.
    UnicodeRange_LatinExt2 = 2,
    
    // U+0300...U+0400 - Combining Diacritical Marks, Greek and Coptic.
    UnicodeRange_Greek = 3,
    
    // U+0400...U+0500 - Cyrillic.
    UnicodeRange_Cyrillic = 4,
    
    // U+0500...U+0600 - Cyrillic Supplement, Hebrew.
    UnicodeRange_Hebrew = 5,
    
    // U+2000...U+2100 - General Punctuation, Superscripts and Subscripts, Currency Symbols, Combining Diacritical Marks for Symbols.
    UnicodeRange_Symbols1 = 6,
    
    // U+2100...U+2200 - Letterlike Symbols, Number Forms, Arrows.
    UnicodeRange_Symbols2 = 7,

    // U+2200...U+2300 - Letterlike Symbols, Number Forms, Arrows.
    UnicodeRange_Math = 8,
    
    // U+2500...U+2600 - Box Drawing, Block Elements, Geometric Shapes.
    UnicodeRange_Drawing = 9,
    
    // For looping.
    UnicodeRange_Min = 0,
    UnicodeRange_Max = 9,
    UnicodeRange_Count = 10
};

bool CharacterIsSupported(uint32_t c);

UnicodeRangeEnum UnicodeRangeForCharacter(uint32_t c);

int OffsetForUnicodeRange(UnicodeRangeEnum range);

const char* NameForUnicodeRange(UnicodeRangeEnum range);

uint32_t CharToGlyphIndex(uint32_t c);

#endif
